@@ -1,5 +1,5 @@
 // ClientPattern.cc for Fluxbox Window Manager
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ClientPattern.cc 3903 2005-02-13 16:34:37Z fluxgen $
+// $Id: ClientPattern.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "ClientPattern.hh"
 #include "RegExp.hh"
@@ -169,7 +169,13 @@ std::string ClientPattern::toString() const {
     Terms::const_iterator it = m_terms.begin();
     Terms::const_iterator it_end = m_terms.end();
     for (; it != it_end; ++it) {
+
         pat.append(" (");
+        // replace special chars like ( ) and [ ] with \( \) and \[ \]
+        string orig = FbTk::StringUtil::replaceString((*it)->orig, "(", "\\(");
+        orig = FbTk::StringUtil::replaceString(orig, ")", "\\)");
+        orig = FbTk::StringUtil::replaceString(orig, "[", "\\[");
+        orig = FbTk::StringUtil::replaceString(orig, "]", "\\]");
 
         switch ((*it)->prop) {
         case NAME:
@@ -185,7 +191,7 @@ std::string ClientPattern::toString() const {
             pat.append("role=");
         }
 
-        pat.append((*it)->orig);
+        pat.append(orig);
         pat.append(")");
     }
 
