@@ -1,5 +1,5 @@
 // Ewmh.cc for fluxbox
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Ewmh.cc 3994 2005-05-06 15:24:53Z simonb $
+// $Id: Ewmh.cc 4231 2006-03-09 18:33:36Z mathias $
 
 #include "Ewmh.hh"
 
@@ -27,8 +27,8 @@
 #include "Window.hh"
 #include "WinClient.hh"
 #include "Workspace.hh"
-#include "fluxbox.hh"
-
+#include "Layer.hh"
+#include "FbTk/App.hh"
 #include "FbTk/FbWindow.hh"
 #include "FbTk/I18n.hh"
 
@@ -194,7 +194,7 @@ void Ewmh::setupFrame(FluxboxWindow &win) {
                              &data);
     if (data) {
         Atom *atoms = (unsigned long *)data;
-        for (unsigned long l=0; l<nitems; ++l) {
+        for (unsigned long l = 0; l < nitems; ++l) {
             /* From Extended Window Manager Hints, draft 1.3:
              *
              * _NET_WM_WINDOW_TYPE_DOCK indicates a dock or panel feature.
@@ -217,7 +217,7 @@ void Ewmh::setupFrame(FluxboxWindow &win) {
 
                 win.setFocusHidden(true);
                 win.setIconHidden(true);
-                win.moveToLayer(Fluxbox::instance()->getDesktopLayer());
+                win.moveToLayer(Layer::DESKTOP);
                 win.setDecorationMask(0);
                 win.setTabable(false);
                 win.setMovable(false);
@@ -241,7 +241,8 @@ void Ewmh::setupFrame(FluxboxWindow &win) {
             win.setWorkspace(desktop);
 
         XFree(data);
-    }
+    } else 
+        updateWorkspace(win);
 
 }
 
@@ -265,6 +266,10 @@ void Ewmh::updateFocusedWindow(BScreen &screen, Window win) {
                                        (unsigned char *)&win, 1);
 }
 
+void Ewmh::updateClientClose(WinClient &winclient){
+    updateClientList(winclient.screen());
+}
+
 void Ewmh::updateClientList(BScreen &screen) {
     size_t num=0;
 
@@ -283,8 +288,8 @@ void Ewmh::updateClientList(BScreen &screen) {
 
     }
     // and count icons
-    BScreen::Icons::const_iterator icon_it = screen.getIconList().begin();
-    BScreen::Icons::const_iterator icon_it_end = screen.getIconList().end();
+    BScreen::Icons::const_iterator icon_it = screen.iconList().begin();
+    BScreen::Icons::const_iterator icon_it_end = screen.iconList().end();
     for (; icon_it != icon_it_end; ++icon_it) {
         num += (*icon_it)->numClients();
     }
@@ -322,7 +327,7 @@ void Ewmh::updateClientList(BScreen &screen) {
     }
 
     // plus iconified windows
-    icon_it = screen.getIconList().begin();
+    icon_it = screen.iconList().begin();
     for (; icon_it != icon_it_end; ++icon_it) {
         FluxboxWindow::ClientList::iterator client_it = (*icon_it)->clientList().begin();
         FluxboxWindow::ClientList::iterator client_it_end = (*icon_it)->clientList().end();
@@ -432,7 +437,7 @@ void Ewmh::updateWorkspaceCount(BScreen &screen) {
      * Window Manager to indicate the number of virtual
      * desktops.
      */
-    unsigned long numworkspaces = screen.getCount();
+    unsigned long numworkspaces = screen.numberOfWorkspaces();
     screen.rootWindow().changeProperty(m_net_number_of_desktops,
                                        XA_CARDINAL, 32,
                                        PropModeReplace,
@@ -497,8 +502,8 @@ void Ewmh::updateWorkarea(BScreen &screen) {
      * Not sure how to handle xinerama stuff here.
      * So i'm just doing this on the first head.
      */
-    unsigned long *coords = new unsigned long[4*screen.getCount()];
-    for (unsigned int i=0; i<screen.getCount()*4; i+=4) {
+    unsigned long *coords = new unsigned long[4*screen.numberOfWorkspaces()];
+    for (unsigned int i=0; i < screen.numberOfWorkspaces()*4; i+=4) {
         // x, y
         coords[i] = screen.maxLeft(0);
         coords[i + 1] = screen.maxTop(0);
@@ -511,7 +516,7 @@ void Ewmh::updateWorkarea(BScreen &screen) {
                                        XA_CARDINAL, 32,
                                        PropModeReplace,
                                        (unsigned char *)coords,
-                                       4*screen.getCount());
+                                       4 * screen.numberOfWorkspaces());
 
     delete[] coords;
 }
@@ -521,20 +526,22 @@ void Ewmh::updateState(FluxboxWindow &win) {
     // TODO: should we update the _NET_WM_ALLOWED_ACTIONS
     //       here too?
     
-    std::vector<unsigned int> state;
+    typedef std::vector<unsigned int> StateVec;
+
+    StateVec state;
 
     if (win.isStuck())
         state.push_back(m_net_wm_state_sticky);
     if (win.isShaded())
         state.push_back(m_net_wm_state_shaded);
-    if (win.layerNum() == Fluxbox::instance()->getBottomLayer())
+    if (win.layerNum() == Layer::BOTTOM)
         state.push_back(m_net_wm_state_below);
-    if (win.layerNum() == Fluxbox::instance()->getAboveDockLayer())
+    if (win.layerNum() == Layer::ABOVE_DOCK)
         state.push_back(m_net_wm_state_above);
-    if (win.isFocusHidden()) {
+    if (win.isIconic())
         state.push_back(m_net_wm_state_hidden);
+    if (win.isIconHidden())
         state.push_back(m_net_wm_state_skip_taskbar);
-    }
     if (win.isFullscreen()) {
         state.push_back(m_net_wm_state_fullscreen);
     }
@@ -544,9 +551,31 @@ void Ewmh::updateState(FluxboxWindow &win) {
     
     it = win.clientList().begin();
     for (; it != it_end; ++it) {
-        if (!state.empty())
+        
+        // search the old states for _NET_WM_STATE_SKIP_PAGER and append it
+        // to the current state, so it wont get deleted by us.
+        StateVec client_state(state);
+        Atom ret_type;
+        int fmt;
+        unsigned long nitems, bytes_after;
+        unsigned char *data = 0;
+
+        (*it)->property(m_net_wm_state, 0, 0x7fffffff, False, XA_ATOM,
+                                 &ret_type, &fmt, &nitems, &bytes_after,
+                                 &data);
+        if (data) {
+            Atom *old_states = (Atom *)data;
+            for (unsigned long i=0; i < nitems; ++i)
+                if (old_states[i] == m_net_wm_state_skip_pager) {
+                    client_state.push_back(m_net_wm_state_skip_pager);
+                }
+            XFree(data);
+        }
+
+        if (!client_state.empty())
             (*it)->changeProperty(m_net_wm_state, XA_ATOM, 32, PropModeReplace,
-                                  reinterpret_cast<unsigned char*>(&state.front()), state.size());
+                                  reinterpret_cast<unsigned char*>(&client_state.front()), 
+                                  client_state.size());
         else
             (*it)->deleteProperty(m_net_wm_state);
     }
@@ -568,7 +597,8 @@ void Ewmh::updateHints(FluxboxWindow &win) {
 }
 
 void Ewmh::updateWorkspace(FluxboxWindow &win) {
-    long workspace = win.workspaceNumber();
+    long workspace = win.isInitialized() ? win.workspaceNumber() : win.screen().currentWorkspaceID();
+
     if (win.isStuck())
         workspace = 0xFFFFFFFF; // appear on all desktops/workspaces
 
@@ -608,7 +638,7 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
         // which doesn't apply here (so borrow the variable :) )
         screen = &fbwin->screen();
         // valid workspace number?
-        if (static_cast<unsigned int>(ce.data.l[0]) < screen->getCount())
+        if (static_cast<unsigned int>(ce.data.l[0]) < screen->numberOfWorkspaces())
             screen->sendToWorkspace(ce.data.l[0], fbwin, false);
 
         return true;
@@ -638,20 +668,20 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
 
         // no need to alter number of desktops if they are the same
         // or if requested number of workspace is less than zero
-        if (screen->getCount() == static_cast<unsigned int>(ce.data.l[0]) ||
+        if (screen->numberOfWorkspaces() == static_cast<unsigned int>(ce.data.l[0]) ||
             ce.data.l[0] < 0)
             return true;
 
-        if (screen->getCount() > static_cast<unsigned int>(ce.data.l[0])) {
+        if (screen->numberOfWorkspaces() > static_cast<unsigned int>(ce.data.l[0])) {
             // remove last workspace until we have
             // the same number of workspaces
-            while (screen->getCount() != static_cast<unsigned int>(ce.data.l[0])) {
+            while (screen->numberOfWorkspaces() != static_cast<unsigned int>(ce.data.l[0])) {
                 screen->removeLastWorkspace();
-                if (screen->getCount() == 1) // must have at least one workspace
+                if (screen->numberOfWorkspaces() == 1) // must have at least one workspace
                     break;
             }
         } else { // add workspaces to screen until workspace count match the requested size
-            while (screen->getCount() != static_cast<unsigned int>(ce.data.l[0])) {
+            while (screen->numberOfWorkspaces() != static_cast<unsigned int>(ce.data.l[0])) {
                 screen->addWorkspace();
             }
         }
@@ -663,7 +693,7 @@ bool Ewmh::checkClientMessage(const XClientMessageEvent &ce,
         // ce.data.l[0] = workspace number
 
         // prevent out of range value
-        if (static_cast<unsigned int>(ce.data.l[0]) >= screen->getCount())
+        if (static_cast<unsigned int>(ce.data.l[0]) >= screen->numberOfWorkspaces())
             return true;
         screen->changeWorkspaceID(ce.data.l[0]);
         return true;
@@ -771,6 +801,7 @@ void Ewmh::createAtoms() {
     m_net_wm_state_fullscreen = XInternAtom(disp, "_NET_WM_STATE_FULLSCREEN", False);
     m_net_wm_state_hidden = XInternAtom(disp, "_NET_WM_STATE_HIDDEN", False);
     m_net_wm_state_skip_taskbar = XInternAtom(disp, "_NET_WM_STATE_SKIP_TASKBAR", False);
+    m_net_wm_state_skip_pager = XInternAtom(disp, "_NET_WM_STATE_SKIP_PAGER", False);
     m_net_wm_state_above = XInternAtom(disp, "_NET_WM_STATE_ABOVE", False);
     m_net_wm_state_below = XInternAtom(disp, "_NET_WM_STATE_BELOW", False);
 
@@ -854,22 +885,24 @@ void Ewmh::setState(FluxboxWindow &win, Atom state, bool value) {
         if ((value && !win.isFullscreen()) ||
             (!value && win.isFullscreen()))
         setFullscreen(win, value);
-    } else if (state == m_net_wm_state_hidden ||
-               state == m_net_wm_state_skip_taskbar) {
-        win.setFocusHidden(value);
+    } else if (state == m_net_wm_state_hidden) { // minimized
+        if (value && !win.isIconic())
+            win.iconify();
+        else if (!value && win.isIconic())
+            win.deiconify();
+    } else if (state == m_net_wm_state_skip_taskbar) {
         win.setIconHidden(value);
     } else if (state == m_net_wm_state_below) {  // bottom layer
         if (value)
-            win.moveToLayer(Fluxbox::instance()->getBottomLayer());
+            win.moveToLayer(Layer::BOTTOM);
         else
-            win.moveToLayer(Fluxbox::instance()->getNormalLayer());
+            win.moveToLayer(Layer::NORMAL);
 
     } else if (state == m_net_wm_state_above) { // above layer
         if (value)
-            win.moveToLayer(Fluxbox::instance()->getAboveDockLayer());
+            win.moveToLayer(Layer::ABOVE_DOCK);
         else
-            win.moveToLayer(Fluxbox::instance()->getNormalLayer());
-
+            win.moveToLayer(Layer::NORMAL);
     }
 }
 
@@ -885,21 +918,24 @@ void Ewmh::toggleState(FluxboxWindow &win, Atom state) {
         win.maximizeVertical();
     } else if (state == m_net_wm_state_fullscreen) { // fullscreen
         setFullscreen(win, getState(win) == 0); // toggle current state
-    } else if (state == m_net_wm_state_hidden ||
-               state == m_net_wm_state_skip_taskbar) {
-        win.setFocusHidden(!win.isFocusHidden());
+    } else if (state == m_net_wm_state_hidden) { // minimized
+        if(win.isIconic())
+            win.deiconify();
+        else
+            win.iconify();
+    } else if (state == m_net_wm_state_skip_taskbar) {
         win.setIconHidden(!win.isIconHidden());
     } else if (state == m_net_wm_state_below) { // bottom layer
-        if (win.layerNum() == Fluxbox::instance()->getBottomLayer())
-            win.moveToLayer(Fluxbox::instance()->getNormalLayer());
+        if (win.layerNum() == Layer::BOTTOM)
+            win.moveToLayer(Layer::NORMAL);
         else
-            win.moveToLayer(Fluxbox::instance()->getBottomLayer());
+            win.moveToLayer(Layer::BOTTOM);
 
     } else if (state == m_net_wm_state_above) { // top layer
-        if (win.layerNum() == Fluxbox::instance()->getAboveDockLayer())
-            win.moveToLayer(Fluxbox::instance()->getNormalLayer());
+        if (win.layerNum() == Layer::ABOVE_DOCK)
+            win.moveToLayer(Layer::NORMAL);
         else
-            win.moveToLayer(Fluxbox::instance()->getAboveDockLayer());
+            win.moveToLayer(Layer::ABOVE_DOCK);
     }
 
 }
