@@ -1,5 +1,5 @@
 // Color.cc for Fluxbox window manager
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Color.cc 3864 2005-01-24 18:02:34Z mathias $
+// $Id: Color.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "Color.hh"
 
@@ -31,11 +31,9 @@
 using namespace std;
 
 namespace {
-unsigned char maxValue(unsigned short colval) {
-    if (colval == 65535) 
-        return 0xFF;
 
-    return static_cast<unsigned char>(colval/0xFF);
+inline unsigned char maxValue(unsigned short colval) {
+   return colval == 65535 ? 0xFF : static_cast<unsigned char>(colval/0xFF);
 }
 
 };
@@ -106,6 +104,17 @@ bool Color::setFromString(const char *color_string, int screen) {
     return true;
 }
 
+bool Color::validColorString(const char *color_string, int screen) {
+    XColor color;
+    Display *disp = App::instance()->display();
+    Colormap colm = DefaultColormap(disp, screen);   
+    // trim white space
+    string color_string_tmp = color_string;
+    StringUtil::removeFirstWhitespace(color_string_tmp);
+    StringUtil::removeTrailingWhitespace(color_string_tmp);
+
+    return XParseColor(disp, colm, color_string_tmp.c_str(), &color) != 0;
+}
 
 Color &Color::operator  = (const Color &col_copy) {
     // check for aliasing
