@@ -1,5 +1,5 @@
 // Resource.cc
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen(at)users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,13 +19,18 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Resource.cc,v 1.5 2004/01/19 18:26:04 fluxgen Exp $
+// $Id: Resource.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "XrmDatabaseHelper.hh"
 #include "Resource.hh"
+#include "I18n.hh"
 
 #include <iostream>
-#include <cassert>
+#ifdef HAVE_CASSERT
+  #include <cassert>
+#else
+  #include <assert.h>
+#endif
 
 using namespace std;
 
@@ -81,8 +86,9 @@ bool ResourceManager::load(const char *filename) {
                            resource->altName().c_str(), &value_type, &value))			
             resource->setFromString(value.addr);
         else {
-            cerr<<"Failed to read: "<<resource->name()<<endl;
-            cerr<<"Setting default value"<<endl;
+            _FB_USES_NLS;
+            cerr<<_FBTKTEXT(Error, FailedRead, "Failed to read", "Couldn't load a resource (following)")<<": "<<resource->name()<<endl;
+            cerr<<_FBTKTEXT(Error, UsingDefault, "Setting default value", "Falling back to default value for resource")<<endl;
             resource->setDefaultValue();
         }
     }
@@ -155,8 +161,20 @@ Resource_base *ResourceManager::findResource(const std::string &resname) {
     return 0;
 }
 
-string ResourceManager::resourceValue(const std::string &resname) {
-    Resource_base *res = findResource(resname);
+const Resource_base *ResourceManager::findResource(const std::string &resname) const {
+   // find resource name
+    ResourceList::const_iterator i = m_resourcelist.begin();
+    ResourceList::const_iterator i_end = m_resourcelist.end();
+    for (; i != i_end; ++i) {
+        if ((*i)->name() == resname ||
+            (*i)->altName() == resname) 
+            return *i;
+    }
+    return 0;
+}
+
+string ResourceManager::resourceValue(const std::string &resname) const {
+    const Resource_base *res = findResource(resname);
     if (res != 0)
         return res->getString();
 
