@@ -1,5 +1,5 @@
 // Gnome.cc for fluxbox
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,17 +19,19 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Gnome.cc 4015 2005-05-13 11:01:01Z simonb $
+// $Id: Gnome.cc 4221 2006-02-20 21:04:35Z fluxgen $
 
 #include "Gnome.hh"
 
-#include "fluxbox.hh"
+#include "App.hh"
 #include "Window.hh"
 #include "Screen.hh"
 #include "WinClient.hh"
 #include "Workspace.hh"
+#include "Layer.hh"
 #include "FbTk/I18n.hh"
 
+
 #include <iostream>
 #include <new>
 using namespace std;
@@ -234,13 +236,13 @@ void Gnome::updateCurrentWorkspace(BScreen &screen) {
 }
 
 void Gnome::updateWorkspaceCount(BScreen &screen) {
-    long numworkspaces = screen.getCount();
+    long numworkspaces = screen.numberOfWorkspaces();
     screen.rootWindow().changeProperty(m_gnome_wm_win_workspace_count, XA_CARDINAL, 32, PropModeReplace,
                                        (unsigned char *)&numworkspaces, 1);
 }
 
 void Gnome::updateWorkspace(FluxboxWindow &win) {
-    long val = win.workspaceNumber();
+    long val = win.isInitialized() ? win.workspaceNumber() : win.screen().currentWorkspaceID();
     if (win.isStuck()) {
         val = -1;
     }
@@ -279,7 +281,7 @@ void Gnome::updateState(FluxboxWindow &win) {
 void Gnome::updateLayer(FluxboxWindow &win) {
     //TODO - map from flux layers to gnome ones
     // our layers are in the opposite direction to GNOME
-    long layernum = Fluxbox::instance()->getDesktopLayer() - win.layerNum();
+    long layernum = Layer::DESKTOP - win.layerNum();
 
     FluxboxWindow::ClientList::iterator client_it = win.clientList().begin();
     FluxboxWindow::ClientList::iterator client_it_end = win.clientList().end();
@@ -302,12 +304,12 @@ bool Gnome::checkClientMessage(const XClientMessageEvent &ce, BScreen * screen,
 #endif//!DEBUG
         if ( winclient !=0 && // the message sent to client window?
              ce.data.l[0] >= 0 &&
-             ce.data.l[0] < (signed)winclient->screen().getCount()) {
+             ce.data.l[0] < (signed)winclient->screen().numberOfWorkspaces()) {
             winclient->screen().changeWorkspaceID(ce.data.l[0]);
 					
         } else if (screen!=0 && //the message sent to root window?
                    ce.data.l[0] >= 0 &&
-                   ce.data.l[0] < (signed)screen->getCount())
+                   ce.data.l[0] < (signed)screen->numberOfWorkspaces())
             screen->changeWorkspaceID(ce.data.l[0]);
         return true;
     } else if (winclient == 0)
@@ -418,47 +420,47 @@ void Gnome::setLayer(FluxboxWindow *win, int layer) {
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", WIN_LAYER_DESKTOP)"<<endl;
 #endif // DEBUG
-        layer = Fluxbox::instance()->getDesktopLayer();
+        layer = Layer::DESKTOP;
         break;
     case WIN_LAYER_BELOW:
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", WIN_LAYER_BELOW)"<<endl;
 #endif // DEBUG
-        layer = Fluxbox::instance()->getBottomLayer();
+        layer = Layer::BOTTOM;
         break;
     case WIN_LAYER_NORMAL:
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", WIN_LAYER_NORMAL)"<<endl;
 #endif // DEBUG
-        layer = Fluxbox::instance()->getNormalLayer();
+        layer = Layer::NORMAL;
         break;		
     case WIN_LAYER_ONTOP:
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", WIN_LAYER_ONTOP)"<<endl;
 #endif // DEBUG
-        layer = Fluxbox::instance()->getTopLayer();
+        layer = Layer::TOP;
         break;
     case WIN_LAYER_DOCK:
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", WIN_LAYER_DOCK)"<<endl;
 #endif // DEBUG
-        layer = Fluxbox::instance()->getDockLayer();
+        layer = Layer::DOCK;
         break;
     case WIN_LAYER_ABOVE_DOCK:
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", WIN_LAYER_ABOVE_DOCK)"<<endl;
 #endif // DEBUG
-        layer = Fluxbox::instance()->getAboveDockLayer();
+        layer = Layer::ABOVE_DOCK;
         break;
     case WIN_LAYER_MENU:
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", WIN_LAYER_MENU)"<<endl;
 #endif // DEBUG
-        layer = Fluxbox::instance()->getMenuLayer();
+        layer = Layer::MENU;
         break;
     default:
         // our windows are in the opposite direction to gnome
-        layer = Fluxbox::instance()->getDesktopLayer() - layer;
+        layer = Layer::DESKTOP - layer;
 #ifdef DEBUG
         cerr<<"Gnome::setLayer("<<win->title()<<", "<<layer<<")"<<endl;
 #endif // DEBUG
