@@ -1,5 +1,5 @@
 // IconbarTool.cc
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: IconbarTool.cc 3974 2005-04-29 16:54:30Z mathias $
+// $Id: IconbarTool.cc 4229 2006-03-07 17:17:52Z fluxgen $
 
 #include "IconbarTool.hh"
 
@@ -29,11 +29,13 @@
 #include "Window.hh"
 #include "IconButton.hh"
 #include "Workspace.hh"
-#include "fluxbox.hh"
 #include "FbMenu.hh"
 #include "BoolMenuItem.hh"
 #include "CommandParser.hh"
 #include "WinClient.hh"
+#include "FocusControl.hh"
+#include "FbCommands.hh"
+#include "Layer.hh"
 
 #include "FbTk/I18n.hh"
 #include "FbTk/Menu.hh"
@@ -49,6 +51,8 @@
 #include <iterator>
 using namespace std;
 
+namespace FbTk {
+
 template<>
 void FbTk::Resource<IconbarTool::Mode>::setFromString(const char *strval) {
     if (strcasecmp(strval, "None") == 0) 
@@ -87,7 +91,7 @@ void FbTk::Resource<IconbarTool::DeiconifyMode>::setFromString(const char* strva
 }
 
 template<>
-std::string FbTk::Resource<IconbarTool::DeiconifyMode>::getString() {
+std::string FbTk::Resource<IconbarTool::DeiconifyMode>::getString() const {
     switch (m_value) {
     case IconbarTool::SEMIFOLLOW:
         return std::string("SemiFollow");
@@ -119,7 +123,7 @@ void FbTk::Resource<IconbarTool::WheelMode>::setFromString(const char* strval) {
 
 
 template<>
-std::string FbTk::Resource<IconbarTool::WheelMode>::getString() {
+std::string FbTk::Resource<IconbarTool::WheelMode>::getString() const {
     switch(m_value) {
     case IconbarTool::ON:
         return std::string("On");
@@ -137,7 +141,7 @@ void FbTk::Resource<Container::Alignment>::setDefaultValue() {
 }
 
 template<>
-string FbTk::Resource<Container::Alignment>::getString() {
+string FbTk::Resource<Container::Alignment>::getString() const {
     switch (m_value) {
     case Container::LEFT:
         return string("Left");
@@ -162,7 +166,7 @@ void FbTk::Resource<Container::Alignment>::setFromString(const char *str) {
 }
 
 template<>
-string FbTk::Resource<IconbarTool::Mode>::getString() {
+string FbTk::Resource<IconbarTool::Mode>::getString() const {
 
     switch (m_value) {
     case IconbarTool::NONE:
@@ -190,6 +194,7 @@ string FbTk::Resource<IconbarTool::Mode>::getString() {
     // default string
     return string("Icons");
 }
+} // end namespace FbTk
 
 namespace {
 
@@ -235,9 +240,7 @@ void setupModeMenu(FbTk::Menu &menu, IconbarTool &handler) {
 
     menu.setLabel(_FBTEXT(Toolbar, IconbarMode, "Iconbar Mode", "Menu title - chooses which set of icons are shown in the iconbar"));
 
-    RefCount<Command> saverc_cmd(new SimpleCommand<Fluxbox>(
-                                                            *Fluxbox::instance(), 
-                                                            &Fluxbox::save_rc));
+    RefCount<Command> saverc_cmd(new FbCommands::SaveResources());
     
 
     menu.insert(new ToolbarModeMenuItem(_FBTEXT(Toolbar, IconbarModeNone, 
@@ -381,7 +384,7 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
     m_rc_use_pixmap(screen.resourceManager(), true,
                     screen.name() + ".iconbar.usePixmap", screen.altName() + ".Iconbar.UsePixmap"),
     m_menu(screen.menuTheme(), screen.imageControl(),
-           *screen.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())) {
+           *screen.layerManager().getLayer(Layer::MENU)) {
 
     // setup mode menu
     setupModeMenu(m_menu, *this);
@@ -721,8 +724,8 @@ void IconbarTool::renderTheme() {
                                                              m_icon_container.height(),
                                                              m_theme.unfocusedTexture());
         m_unfocused_err_pm = m_screen.imageControl().renderImage(icon_width+1,
-                                                             m_icon_container.height(),
-                                                             m_theme.unfocusedTexture());
+                                                                 m_icon_container.height(),
+                                                                 m_theme.unfocusedTexture());
     }
     if (tmp)
         m_screen.imageControl().removeImage(tmp);
@@ -877,8 +880,8 @@ void IconbarTool::addWindow(FluxboxWindow &win) {
 void IconbarTool::updateIcons() {
     std::list<FluxboxWindow *> itemlist;
     // add icons to the itemlist    
-    BScreen::Icons::iterator icon_it = m_screen.getIconList().begin();
-    BScreen::Icons::iterator icon_it_end = m_screen.getIconList().end();
+    BScreen::Icons::iterator icon_it = m_screen.iconList().begin();
+    BScreen::Icons::iterator icon_it_end = m_screen.iconList().end();
     for (; icon_it != icon_it_end; ++icon_it) {
         if (mode() == ICONS)
             itemlist.push_back(*icon_it);
@@ -904,10 +907,11 @@ void IconbarTool::updateWorkspace() {
 
     // add icons from current workspace
     if (mode() != WORKSPACENOICONS) {
-        BScreen::Icons::iterator icon_it =  m_screen.getIconList().begin();
-        BScreen::Icons::iterator icon_it_end =  m_screen.getIconList().end();
+        BScreen::Icons::iterator icon_it =  m_screen.iconList().begin();
+        BScreen::Icons::iterator icon_it_end =  m_screen.iconList().end();
         for (; icon_it != icon_it_end; ++icon_it) {
-            if ((*icon_it)->workspaceNumber() == m_screen.currentWorkspaceID())
+            if ((*icon_it)->workspaceNumber() == m_screen.currentWorkspaceID() ||
+                (*icon_it)->isStuck())
                 itemlist.push_back(*icon_it);
         }
     }
@@ -930,8 +934,8 @@ void IconbarTool::updateAllWindows() {
     // add icons
     if(mode() != NOICONS && mode() != WORKSPACENOICONS) {
         full_list.insert(full_list.end(),
-                         m_screen.getIconList().begin(),
-                         m_screen.getIconList().end());
+                         m_screen.iconList().begin(),
+                         m_screen.iconList().end());
     }
 
     removeDuplicate(m_icon_list, full_list);
@@ -948,7 +952,7 @@ void IconbarTool::addList(std::list<FluxboxWindow *> &winlist) {
 }
 
 void IconbarTool::timedRender() {
-    WinClient *client = Fluxbox::instance()->getFocusedWindow();
+    WinClient *client = FocusControl::focusedWindow();
     if (client == 0 || client->fbwindow() == 0)
         return;
 
