@@ -1,5 +1,5 @@
 // Resources.cc for Fluxbox Window Manager
-// Copyright (c) 2004 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2004 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,23 +19,29 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Resources.cc 3995 2005-05-06 18:04:14Z mathias $
+// $Id: Resources.cc 4247 2006-03-18 22:02:21Z fluxgen $
 
 
 // holds main resource functions
 
-#include "fluxbox.hh"
 #include "FbTk/StringUtil.hh"
+#include "FbTk/Resource.hh"
+#include "WinButton.hh"
+
+#include "fluxbox.hh"
+
+#include "Layer.hh"
 
 #include <stdio.h>
 #include <string>
-
+#include <vector>
 using namespace std;
 using namespace FbTk;
 
 //-----------------------------------------------------------------
 //---- accessors for int, bool, and some enums with Resource ------
 //-----------------------------------------------------------------
+namespace FbTk {
 
 template<>
 void FbTk::Resource<int>::
@@ -58,7 +64,7 @@ setFromString(char const *strval) {
 }
 
 template<>
-void FbTk::Resource<Fluxbox::TitlebarList>::
+void FbTk::Resource<std::vector<WinButton::Type> >::
 setFromString(char const *strval) {
     vector<std::string> val;
     StringUtil::stringtok(val, strval);
@@ -68,17 +74,17 @@ setFromString(char const *strval) {
 		
     for (int i=0; i<size; i++) {
         if (strcasecmp(val[i].c_str(), "Maximize")==0)
-            m_value.push_back(Fluxbox::MAXIMIZE);
+            m_value.push_back(WinButton::MAXIMIZE);
         else if (strcasecmp(val[i].c_str(), "Minimize")==0)
-            m_value.push_back(Fluxbox::MINIMIZE);
+            m_value.push_back(WinButton::MINIMIZE);
         else if (strcasecmp(val[i].c_str(), "Shade")==0)
-            m_value.push_back(Fluxbox::SHADE);
+            m_value.push_back(WinButton::SHADE);
         else if (strcasecmp(val[i].c_str(), "Stick")==0)
-            m_value.push_back(Fluxbox::STICK);
+            m_value.push_back(WinButton::STICK);
         else if (strcasecmp(val[i].c_str(), "MenuIcon")==0)
-            m_value.push_back(Fluxbox::MENUICON);
+            m_value.push_back(WinButton::MENUICON);
         else if (strcasecmp(val[i].c_str(), "Close")==0)
-            m_value.push_back(Fluxbox::CLOSE);
+            m_value.push_back(WinButton::CLOSE);
     }
 }
 
@@ -111,13 +117,13 @@ setFromString(const char *strval) {
 //-----------------------------------------------------------------
 template<>
 std::string FbTk::Resource<bool>::
-getString() {				
+getString() const {
     return std::string(**this == true ? "true" : "false");
 }
 
 template<>
 std::string FbTk::Resource<int>::
-getString() {
+getString() const {
     char strval[256];
     sprintf(strval, "%d", **this);
     return std::string(strval);
@@ -125,32 +131,32 @@ getString() {
 
 template<>
 std::string FbTk::Resource<std::string>::
-getString() { return **this; }
+getString() const { return **this; }
 
 
 template<>
-std::string FbTk::Resource<Fluxbox::TitlebarList>::
-getString() {
+std::string FbTk::Resource<std::vector<WinButton::Type> >::
+getString() const {
     string retval;
     int size=m_value.size();
     for (int i=0; i<size; i++) {
         switch (m_value[i]) {
-        case Fluxbox::SHADE:
+        case WinButton::SHADE:
             retval.append("Shade");
             break;
-        case Fluxbox::MINIMIZE:
+        case WinButton::MINIMIZE:
             retval.append("Minimize");
             break;
-        case Fluxbox::MAXIMIZE:
+        case WinButton::MAXIMIZE:
             retval.append("Maximize");
             break;
-        case Fluxbox::CLOSE:
+        case WinButton::CLOSE:
             retval.append("Close");
             break;
-        case Fluxbox::STICK:
+        case WinButton::STICK:
             retval.append("Stick");
             break;
-        case Fluxbox::MENUICON:
+        case WinButton::MENUICON:
             retval.append("MenuIcon");
             break;
         default:
@@ -164,7 +170,7 @@ getString() {
 
 template<>
 std::string FbTk::Resource<Fluxbox::TabsAttachArea>::
-getString() {
+getString() const {
     if (m_value == Fluxbox::ATTACH_AREA_TITLEBAR)
         return "Titlebar";
     else
@@ -173,7 +179,7 @@ getString() {
 
 template<>
 string FbTk::Resource<unsigned int>::
-getString() {
+getString() const {
     char tmpstr[128];
     sprintf(tmpstr, "%ul", m_value);
     return string(tmpstr);
@@ -181,61 +187,62 @@ getString() {
 
 template<>
 string FbTk::Resource<long long>::
-getString() {
+getString() const {
     char tmpstr[128];
     sprintf(tmpstr, "%llu", (unsigned long long) m_value);
     return string(tmpstr);
 }
 
 template<>
-void FbTk::Resource<Fluxbox::Layer>::
+void FbTk::Resource<Layer>::
 setFromString(const char *strval) {
     int tempnum = 0;
     if (sscanf(strval, "%d", &tempnum) == 1)
         m_value = tempnum;
     else if (strcasecmp(strval, "Menu") == 0)
-        m_value = Fluxbox::instance()->getMenuLayer();
+        m_value = ::Layer::MENU;
     else if (strcasecmp(strval, "AboveDock") == 0)
-        m_value = Fluxbox::instance()->getAboveDockLayer();
+        m_value = ::Layer::ABOVE_DOCK;
     else if (strcasecmp(strval, "Dock") == 0)
-        m_value = Fluxbox::instance()->getDockLayer();
+        m_value = ::Layer::DOCK;
     else if (strcasecmp(strval, "Top") == 0)
-        m_value = Fluxbox::instance()->getTopLayer();
+        m_value = ::Layer::TOP;
     else if (strcasecmp(strval, "Normal") == 0)
-        m_value = Fluxbox::instance()->getNormalLayer();
+        m_value = ::Layer::NORMAL;
     else if (strcasecmp(strval, "Bottom") == 0)
-        m_value = Fluxbox::instance()->getBottomLayer();
+        m_value = ::Layer::BOTTOM;
     else if (strcasecmp(strval, "Desktop") == 0)
-        m_value = Fluxbox::instance()->getDesktopLayer();
+        m_value = ::Layer::DESKTOP;
     else 
         setDefaultValue();
 }
 
 
 template<>
-string FbTk::Resource<Fluxbox::Layer>::
-getString() {
-
-    if (m_value.getNum() == Fluxbox::instance()->getMenuLayer()) 
-        return string("Menu");
-    else if (m_value.getNum() == Fluxbox::instance()->getAboveDockLayer()) 
-        return string("AboveDock");
-    else if (m_value.getNum() == Fluxbox::instance()->getDockLayer()) 
-        return string("Dock");
-    else if (m_value.getNum() == Fluxbox::instance()->getTopLayer()) 
-        return string("Top");
-    else if (m_value.getNum() == Fluxbox::instance()->getNormalLayer()) 
-        return string("Normal");
-    else if (m_value.getNum() == Fluxbox::instance()->getBottomLayer()) 
-        return string("Bottom");
-    else if (m_value.getNum() == Fluxbox::instance()->getDesktopLayer()) 
-        return string("Desktop");
-    else {
+std::string FbTk::Resource<Layer>::
+getString() const {
+    switch (m_value.getNum()) {
+    case Layer::MENU:
+        return std::string("Menu");
+    case Layer::ABOVE_DOCK:
+        return std::string("AboveDock");
+    case Layer::DOCK:
+        return std::string("Dock");
+    case Layer::TOP:
+        return std::string("Top");
+    case Layer::NORMAL:
+        return std::string("Normal");
+    case Layer::BOTTOM:
+        return std::string("Bottom");
+    case Layer::DESKTOP:
+        return std::string("Desktop");
+    default:
         char tmpstr[128];
         sprintf(tmpstr, "%d", m_value.getNum());
-        return string(tmpstr);
+        return std::string(tmpstr);
     }
 }
+
 template<>
 void FbTk::Resource<long>::
 setFromString(const char *strval) {   
@@ -245,8 +252,9 @@ setFromString(const char *strval) {
  
 template<>
 string FbTk::Resource<long>::
-getString() {
+getString() const {
     char tmpstr[128];
     sprintf(tmpstr, "%ld", m_value);
     return string(tmpstr);
 }
+} // end namespace FbTk
