@@ -1,5 +1,5 @@
 // Toolbar.cc for Fluxbox
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Toolbar.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Toolbar.cc 3990 2005-05-03 13:53:25Z simonb $
+// $Id: Toolbar.cc 4227 2006-03-02 07:49:13Z mathias $
 
 #include "Toolbar.hh"
 
@@ -32,15 +32,20 @@
 // themes
 #include "ToolbarTheme.hh"
 
-#include "FbTk/I18n.hh"
 #include "fluxbox.hh"
 #include "Screen.hh"
 #include "IntResMenuItem.hh"
 #include "BoolMenuItem.hh"
+
+#ifdef XINERAMA
 #include "Xinerama.hh"
+#endif // XINERAMA
+
 #include "Strut.hh"
 #include "CommandParser.hh"
+#include "Layer.hh"
 
+#include "FbTk/I18n.hh"
 #include "FbTk/ImageControl.hh"
 #include "FbTk/MacroCommand.hh"
 #include "FbTk/EventManager.hh"
@@ -73,6 +78,8 @@
 
 using namespace std;
 
+namespace FbTk {
+
 template<>
 void FbTk::Resource<Toolbar::Placement>::
 setFromString(const char *strval) {
@@ -106,7 +113,7 @@ setFromString(const char *strval) {
 
 template<>
 string FbTk::Resource<Toolbar::Placement>::
-getString() {
+getString() const {
     switch (m_value) {
     case Toolbar::TOPLEFT:
         return string("TopLeft");
@@ -148,7 +155,7 @@ getString() {
     //default string
     return string("BottomCenter");
 }
-
+} // end namespace FbTk
 
 namespace {
 class SetToolbarPlacementCmd: public FbTk::Command {
@@ -198,15 +205,15 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::XLayer &layer, size_t width):
     m_layeritem(frame.window, layer),
     m_layermenu(scrn.menuTheme(),
                 scrn.imageControl(),
-                *scrn.layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
+                *scrn.layerManager().getLayer(Layer::MENU),
                 this,
                 true),
     m_placementmenu(scrn.menuTheme(),
                     scrn.imageControl(),
-                    *scrn.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())),
+                    *scrn.layerManager().getLayer(Layer::MENU)),
     m_toolbarmenu(scrn.menuTheme(),
                   scrn.imageControl(),
-                  *scrn.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())),
+                  *scrn.layerManager().getLayer(Layer::MENU)),
     m_theme(scrn.screenNumber()),
     m_tool_factory(scrn),
     m_strut(0),
@@ -220,7 +227,7 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::XLayer &layer, size_t width):
                        scrn.name() + ".toolbar.widthPercent", scrn.altName() + ".Toolbar.WidthPercent"),
     m_rc_alpha(scrn.resourceManager(), 255,
                        scrn.name() + ".toolbar.alpha", scrn.altName() + ".Toolbar.Alpha"),
-    m_rc_layernum(scrn.resourceManager(), Fluxbox::Layer(Fluxbox::instance()->getDesktopLayer()),
+    m_rc_layernum(scrn.resourceManager(), Layer(Layer::DESKTOP),
                   scrn.name() + ".toolbar.layer", scrn.altName() + ".Toolbar.Layer"),
     m_rc_on_head(scrn.resourceManager(), 0,
                  scrn.name() + ".toolbar.onhead", scrn.altName() + ".Toolbar.onHead"),
@@ -302,7 +309,7 @@ void Toolbar::updateStrut() {
     clearStrut();
     // we should request space if we're in autohide mode or
     // if the user dont want to request space for toolbar.
-    if (doAutoHide() || *m_rc_maximize_over) {
+    if (doAutoHide() || *m_rc_maximize_over || ! *m_rc_visible) {
         if (had_strut)
             screen().updateAvailableWorkspaceArea();
         return;
@@ -783,62 +790,60 @@ void Toolbar::setupMenus() {
     typedef RefCount<Command> RefCommand;
     typedef SimpleCommand<Toolbar> ToolbarCommand;
 
-    //!! TODO: this should be inserted by the workspace tool
-
-
-    RefCommand start_edit(CommandParser::instance().parseLine("setworkspacenamedialog"));
-    menu().insert(_FBTEXT(Toolbar, EditWkspcName,
-                          "Edit current workspace name", "Edit current workspace name"),
-                  start_edit);
-
     menu().setLabel(_FBTEXT(Toolbar, Toolbar,
                             "Toolbar", "Title of Toolbar menu"));
 
-    MenuItem *toolbar_menuitem = new IntResMenuItem(_FBTEXT(Toolbar, WidthPercent, "Toolbar width percent", "Percentage of screen width taken by toolbar"),
-                                                    m_rc_width_percent,
-                                                    0, 100, menu()); // min/max value
-
-
     RefCommand reconfig_toolbar(new ToolbarCommand(*this, &Toolbar::reconfigure));
     RefCommand save_resources(CommandParser::instance().parseLine("saverc"));
     MacroCommand *toolbar_menuitem_macro = new MacroCommand();
     toolbar_menuitem_macro->add(reconfig_toolbar);
     toolbar_menuitem_macro->add(save_resources);
-
     RefCommand reconfig_toolbar_and_save_resource(toolbar_menuitem_macro);
-    toolbar_menuitem->setCommand(reconfig_toolbar_and_save_resource);
 
-    menu().insert(toolbar_menuitem);
+    MacroCommand *visible_macro = new MacroCommand();
+    RefCommand toggle_visible(new ToolbarCommand(*this, &Toolbar::updateVisibleState));
+    visible_macro->add(toggle_visible);
+    visible_macro->add(reconfig_toolbar);
+    visible_macro->add(save_resources);
+    RefCommand toggle_visible_cmd(visible_macro);
+    menu().insert(new BoolMenuItem(_FBTEXT(Common, Visible, 
+                                           "Visible", "Whether this item is visible"),
+                                   *m_rc_visible, toggle_visible_cmd));
 
     menu().insert(new BoolMenuItem(_FBTEXT(Common, AutoHide,
                                            "Auto hide", "Toggle auto hide of toolbar"),
                                    *m_rc_auto_hide,
                                    reconfig_toolbar_and_save_resource));
 
-    MacroCommand *visible_macro = new MacroCommand();
-    RefCommand toggle_visible(new ToolbarCommand(*this, &Toolbar::updateVisibleState));
-    visible_macro->add(toggle_visible);
-    visible_macro->add(save_resources);
-    RefCommand toggle_visible_cmd(visible_macro);
-    menu().insert(new BoolMenuItem(_FBTEXT(Common, Visible, "Visible", "Whether this item is visible"),
-                                   *m_rc_visible, toggle_visible_cmd));
+    MenuItem *toolbar_menuitem = 
+        new IntResMenuItem(_FBTEXT(Toolbar, WidthPercent, 
+                                   "Toolbar width percent", 
+                                   "Percentage of screen width taken by toolbar"),
+                           m_rc_width_percent,
+                           0, 100, menu()); // min/max value
 
-    menu().insert(new BoolMenuItem(_FBTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"),
+
+    toolbar_menuitem->setCommand(reconfig_toolbar_and_save_resource);
+    menu().insert(toolbar_menuitem);
+
+    menu().insert(new BoolMenuItem(_FBTEXT(Common, MaximizeOver,
+                                           "Maximize Over", 
+                                           "Maximize over this thing when maximizing"),
                                    *m_rc_maximize_over,
                                    reconfig_toolbar_and_save_resource));
-    menu().insert(_FBTEXT(Menu, Layer, "Layer...", "Title of Layer menu"), &layermenu());
-
-
-
+    menu().insert(_FBTEXT(Menu, Layer, "Layer...", "Title of Layer menu"), &layerMenu());
+#ifdef XINERAMA
     if (screen().hasXinerama()) {
         menu().insert(_FBTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"),
-                      new XineramaHeadMenu<Toolbar>(screen().menuTheme(),
-                                                    screen(),
-                                                    screen().imageControl(),
-                                                    *screen().layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
-                                                    *this,
-                                                    _FBTEXT(Toolbar, OnHead, "Toolbar on Head", "Title of toolbar on head menu")));
+         new XineramaHeadMenu<Toolbar>(screen().menuTheme(),
+                                       screen(),
+                                       screen().imageControl(),
+                                       *screen().layerManager().getLayer(::Layer::MENU),
+                                       *this,
+                                       _FBTEXT(Toolbar, OnHead, "Toolbar on Head", 
+                                               "Title of toolbar on head menu")));
     }
+#endif // XINERAMA
 
     typedef pair<const char*, Toolbar::Placement> PlacementP;
     typedef list<PlacementP> Placements;
