@@ -1,5 +1,5 @@
 // WinButtonTheme.cc for Fluxbox - an X11 Window manager
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WinButtonTheme.cc 3995 2005-05-06 18:04:14Z mathias $
+// $Id: WinButtonTheme.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "WinButtonTheme.hh"
 
@@ -83,6 +83,10 @@ void WinButtonTheme::reconfigTheme() {
     m_maximize_unfocus_pm->scale(size, size);
     m_maximize_pressed_pm->scale(size, size);
 
+    m_menuicon_pm->scale(size, size);
+    m_menuicon_unfocus_pm->scale(size, size);
+    m_menuicon_pressed_pm->scale(size, size);
+
     m_iconify_pm->scale(size, size);
     m_iconify_unfocus_pm->scale(size, size);
     m_iconify_pressed_pm->scale(size, size);
