@@ -1,5 +1,5 @@
 // FbRun.cc
-// Copyright (c) 2002-2003 Henrik Kinnunen (fluxgen<at>users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbRun.cc 3795 2004-12-24 03:02:48Z mathias $
+// $Id: FbRun.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "FbRun.hh"
 
@@ -117,7 +117,7 @@ void FbRun::run(const std::string &command) {
     // fork and execute program
     if (!fork()) {
         setsid();
-        execl("/bin/sh", "/bin/sh", "-c", command.c_str(), 0);
+        execl("/bin/sh", "/bin/sh", "-c", command.c_str(), static_cast<void*>(NULL));
         exit(0); //exit child
     }
 
@@ -223,7 +223,7 @@ void FbRun::redrawLabel() {
 }
 
 void FbRun::keyPressEvent(XKeyEvent &ke) {
-    // strip numlock, capslock and scrolllock mask
+    
     ke.state = FbTk::KeyUtil::instance().cleanMods(ke.state);
 
     int cp= cursorPosition();
@@ -232,10 +232,11 @@ void FbRun::keyPressEvent(XKeyEvent &ke) {
     char keychar[1];
     XLookupString(&ke, keychar, 1, &ks, 0);
     // a modifier key by itself doesn't do anything
-    if (IsModifierKey(ks)) return;
+    if (IsModifierKey(ks)) 
+        return;
 
-    if (ke.state) { // a modifier key is down
-        if (ke.state == ControlMask) {
+    if (FbTk::KeyUtil::instance().isolateModifierMask(ke.state)) { // a modifier key is down
+        if ((ke.state & ControlMask) == ControlMask) {
             switch (ks) {
             case XK_p:
                 prevHistoryItem();
@@ -248,7 +249,7 @@ void FbRun::keyPressEvent(XKeyEvent &ke) {
                 setCursorPosition(cp);
                 break;
             }
-        } else if (ke.state == (Mod1Mask | ShiftMask)) {
+        } else if ((ke.state & (Mod1Mask|ShiftMask)) == (Mod1Mask | ShiftMask)) {
             switch (ks) {
             case XK_less:
                 firstHistoryItem();
@@ -342,7 +343,7 @@ void FbRun::tabCompleteHistory() {
     } else {
         unsigned int nr= 0;
         int history_item = m_current_history_item - 1;
-        string prefix = text().substr(0, cursorPosition());
+        string prefix = text().substr(0, textStartPos() + cursorPosition());
         while (history_item != m_current_history_item && nr++ < m_history.size()) {
             if (history_item <= -1 )
                 history_item= m_history.size() - 1;
@@ -361,7 +362,7 @@ void FbRun::tabCompleteApps() {
   
     static bool first_run= true;
     static string saved_prefix= "";
-    string prefix= text().substr(0, cursorPosition());
+    string prefix= text().substr(0, textStartPos() + cursorPosition());
     FbTk::Directory dir;
 
     bool add_dirs= false;
