@@ -1,5 +1,5 @@
 // main.cc for FbRun
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: main.cc 3867 2005-01-24 18:49:01Z mathias $
+// $Id: main.cc 4199 2006-02-16 06:53:05Z mathias $
 
 #include "FbRun.hh"
 #include "App.hh"
@@ -119,7 +119,7 @@ int main(int argc, char **argv) {
         FbTk::App application(display_name.c_str());
         FbRun fbrun;
 
-        fbrun.setAntialias(antialias);
+        //fbrun.setAntialias(antialias);
 
         if (fontname.size() != 0) {
             if (!fbrun.loadFont(fontname.c_str())) {
@@ -129,7 +129,6 @@ int main(int argc, char **argv) {
         }
 
         // get color
-        XColor xc_foreground, xc_background;
         FbTk::Color fg_color(foreground.c_str(), 0);
         FbTk::Color bg_color(background.c_str(), 0);
 		
@@ -140,8 +139,8 @@ int main(int argc, char **argv) {
             fbrun.resize(fbrun.width(), height);
         if (set_width)
             fbrun.resize(width, fbrun.height());
-        if (antialias)
-            fbrun.setAntialias(antialias);
+        //if (antialias)
+        //    fbrun.setAntialias(antialias);
         // expand and load command history
         string expanded_filename = FbTk::StringUtil::expandFilename(history_file);
         if (!fbrun.loadHistory(expanded_filename.c_str()))
